@@ -1,5 +1,5 @@
 /* Test of xalloc_die() function.
-   Copyright (C) 2009 Free Software Foundation, Inc.
+   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,12 +19,12 @@
 #include <config.h>
 
 #include "xalloc.h"
-
-char *program_name = "test-xalloc-die";
+#include "progname.h"
 
 int
-main (void)
+main (int argc _GL_UNUSED, char **argv)
 {
+  set_program_name (argv[0]);
   xalloc_die ();
   return 0;
 }
