@@ -305,6 +305,8 @@ gsasl_server_finish (Gsasl_session * sctx)
  * gsasl_client_ctx_get:
  * @sctx: libgsasl client handle
  *
+ * Get the libgsasl handle given a libgsasl client handle.
+ *
  * Return value: Returns the libgsasl handle given a libgsasl client handle.
  *
  * Deprecated: This function is not useful with the new 0.2.0 API.
@@ -360,6 +362,8 @@ gsasl_client_application_data_get (Gsasl_session * sctx)
  * gsasl_server_ctx_get:
  * @sctx: libgsasl server handle
  *
+ * Get the libgsasl handle given a libgsasl server handle.
+ *
  * Return value: Returns the libgsasl handle given a libgsasl server handle.
  *
  * Deprecated: This function is not useful with the new 0.2.0 API.
@@ -436,6 +440,8 @@ gsasl_randomize (int strong, char *data, size_t datalen)
  * gsasl_ctx_get:
  * @sctx: libgsasl session handle
  *
+ * Get the libgsasl handle given a libgsasl session handle.
+ *
  * Return value: Returns the libgsasl handle given a libgsasl session handle.
  *
  * Deprecated: This function is not useful with the new 0.2.0 API.
@@ -608,6 +614,9 @@ gsasl_appinfo_get (Gsasl_session * sctx)
  * @mechlist: input character array with SASL mechanism names,
  *   separated by invalid characters (e.g. SPC).
  *
+ * Get name of "best" SASL mechanism supported by the libgsasl server
+ * which is present in the input string.
+ *
  * Return value: Returns name of "best" SASL mechanism supported by
  * the libgsasl server which is present in the input string.
  *
@@ -646,6 +655,9 @@ gsasl_client_callback_authentication_id_set (Gsasl * ctx,
  * gsasl_client_callback_authentication_id_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_client_callback_authentication_id_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_client_callback_authentication_id_set().
  *
@@ -686,6 +698,9 @@ gsasl_client_callback_authorization_id_set (Gsasl * ctx,
  * gsasl_client_callback_authorization_id_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_client_callback_authorization_id_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_client_callback_authorization_id_set().
  *
@@ -726,6 +741,9 @@ gsasl_client_callback_password_set (Gsasl * ctx,
  * gsasl_client_callback_password_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_client_callback_password_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_client_callback_password_set().
  *
@@ -766,6 +784,9 @@ gsasl_client_callback_passcode_set (Gsasl * ctx,
  * gsasl_client_callback_passcode_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_client_callback_passcode_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_client_callback_passcode_set().
  *
@@ -807,6 +828,9 @@ gsasl_client_callback_pin_set (Gsasl * ctx, Gsasl_client_callback_pin cb)
  * gsasl_client_callback_pin_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_client_callback_pin_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_client_callback_pin_set().
  *
@@ -849,6 +873,9 @@ gsasl_client_callback_service_set (Gsasl * ctx,
  * gsasl_client_callback_service_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_client_callback_service_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_client_callback_service_set().
  *
@@ -889,6 +916,9 @@ gsasl_client_callback_anonymous_set (Gsasl * ctx,
  * gsasl_client_callback_anonymous_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_client_callback_anonymous_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_client_callback_anonymous_set().
  *
@@ -927,6 +957,9 @@ gsasl_client_callback_qop_set (Gsasl * ctx, Gsasl_client_callback_qop cb)
  * gsasl_client_callback_qop_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_client_callback_qop_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_client_callback_qop_set().
  *
@@ -969,6 +1002,9 @@ gsasl_client_callback_maxbuf_set (Gsasl * ctx,
  * gsasl_client_callback_maxbuf_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_client_callback_maxbuf_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_client_callback_maxbuf_set().
  *
@@ -1008,6 +1044,9 @@ gsasl_client_callback_realm_set (Gsasl * ctx, Gsasl_client_callback_realm cb)
  * gsasl_client_callback_realm_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_client_callback_realm_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_client_callback_realm_set().
  *
@@ -1048,6 +1087,9 @@ gsasl_server_callback_validate_set (Gsasl * ctx,
  * gsasl_server_callback_validate_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_server_callback_validate_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_server_callback_validate_set().
  *
@@ -1088,6 +1130,9 @@ gsasl_server_callback_retrieve_set (Gsasl * ctx,
  * gsasl_server_callback_retrieve_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_server_callback_retrieve_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_server_callback_retrieve_set().
  *
@@ -1128,6 +1173,9 @@ gsasl_server_callback_cram_md5_set (Gsasl * ctx,
  * gsasl_server_callback_cram_md5_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_server_callback_cram_md5_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_server_callback_cram_md5_set().
  *
@@ -1168,6 +1216,9 @@ gsasl_server_callback_digest_md5_set (Gsasl * ctx,
  * gsasl_server_callback_digest_md5_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_server_callback_digest_md5_set().
+ *
  * Return value: Return the callback earlier set by calling
  * gsasl_server_callback_digest_md5_set().
  *
@@ -1207,6 +1258,9 @@ gsasl_server_callback_external_set (Gsasl * ctx,
  * gsasl_server_callback_external_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_server_callback_external_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_server_callback_external_set().
  *
@@ -1246,6 +1300,9 @@ gsasl_server_callback_anonymous_set (Gsasl * ctx,
  * gsasl_server_callback_anonymous_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_server_callback_anonymous_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_server_callback_anonymous_set().
  *
@@ -1285,6 +1342,9 @@ gsasl_server_callback_realm_set (Gsasl * ctx, Gsasl_server_callback_realm cb)
  * gsasl_server_callback_realm_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_server_callback_realm_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_server_callback_realm_set().
  *
@@ -1325,6 +1385,9 @@ gsasl_server_callback_qop_set (Gsasl * ctx, Gsasl_server_callback_qop cb)
  * gsasl_server_callback_qop_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_server_callback_qop_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_server_callback_qop_set().
  *
@@ -1367,6 +1430,9 @@ gsasl_server_callback_maxbuf_set (Gsasl * ctx,
  * gsasl_server_callback_maxbuf_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_server_callback_maxbuf_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_server_callback_maxbuf_set().
  *
@@ -1408,6 +1474,9 @@ gsasl_server_callback_cipher_set (Gsasl * ctx,
  * gsasl_server_callback_cipher_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_server_callback_cipher_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_server_callback_cipher_set().
  *
@@ -1454,6 +1523,9 @@ gsasl_server_callback_securid_set (Gsasl * ctx,
  * gsasl_server_callback_securid_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_server_callback_securid_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_server_callback_securid_set().
  *
@@ -1496,6 +1568,9 @@ gsasl_server_callback_gssapi_set (Gsasl * ctx,
  * gsasl_server_callback_gssapi_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_server_callback_gssapi_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_server_callback_gssapi_set().
  *
@@ -1537,6 +1612,9 @@ gsasl_server_callback_service_set (Gsasl * ctx,
  * gsasl_server_callback_service_get:
  * @ctx: libgsasl handle.
  *
+ * Get the callback earlier set by calling
+ * gsasl_server_callback_service_set().
+ *
  * Return value: Returns the callback earlier set by calling
  * gsasl_server_callback_service_set().
  *
@@ -1691,54 +1769,36 @@ int
 gsasl_md5pwd_get_password (const char *filename,
 			   const char *username, char *key, size_t * keylen)
 {
-  char matchbuf[BUFSIZ];
-  char line[BUFSIZ];
+  char *tmp;
+  size_t tmplen;
+  int res;
   FILE *fh;
 
   fh = fopen (filename, "r");
   if (fh == NULL)
     return GSASL_FOPEN_ERROR;
+  fclose (fh);
 
-  sprintf (matchbuf, "%s\t", username);
+  res = gsasl_simple_getpass (filename, username, &tmp);
+  if (res != GSASL_OK)
+    return res;
 
-  while (!feof (fh))
+  tmplen = strlen (tmp);
+
+  if (*keylen < tmplen + 1)
     {
-      if (fgets (line, BUFSIZ, fh) == NULL)
-	break;
-
-      if (line[0] == '#')
-	continue;
-
-      while (strlen (line) > 0 && (line[strlen (line) - 1] == '\n' ||
-				   line[strlen (line) - 1] == '\r'))
-	line[strlen (line) - 1] = '\0';
-
-      if (strlen (line) <= strlen (matchbuf))
-	continue;
-
-      if (strncmp (matchbuf, line, strlen (matchbuf)) == 0)
-	{
-	  if (*keylen < strlen (line) - strlen (matchbuf))
-	    {
-	      fclose (fh);
-	      return GSASL_TOO_SMALL_BUFFER;
-	    }
-
-	  *keylen = strlen (line) - strlen (matchbuf);
-
-	  if (key)
-	    memcpy (key, &line[strlen (matchbuf)], *keylen);
-
-	  fclose (fh);
-
-	  return GSASL_OK;
-	}
+      free (tmp);
+      return GSASL_TOO_SMALL_BUFFER;
     }
 
-  if (fclose (fh) != 0)
-    return GSASL_FCLOSE_ERROR;
+  *keylen = tmplen;
 
-  return GSASL_AUTHENTICATION_ERROR;
+  if (key)
+    memcpy (key, tmp, tmplen);
+
+  free (tmp);
+
+  return GSASL_OK;
 }
 
 #include <minmax.h>
@@ -1813,11 +1873,9 @@ gsasl_base64_decode (char const *src, char *target, size_t targsize)
   return copied;
 }
 
-const char *
-_gsasl_obsolete_property_map (Gsasl_session * sctx, Gsasl_property prop)
+static const char *
+pmap (Gsasl_session * sctx, Gsasl_property prop, char *buf, size_t buflen)
 {
-  char buf[BUFSIZ];
-  size_t buflen = BUFSIZ - 1;
   int res;
 
   buf[0] = '\0';
@@ -1979,12 +2037,27 @@ _gsasl_obsolete_property_map (Gsasl_session * sctx, Gsasl_property prop)
   return gsasl_property_fast (sctx, prop);
 }
 
+const char *
+_gsasl_obsolete_property_map (Gsasl_session * sctx, Gsasl_property prop)
+{
+  const char *ret;
+  char *buf;
+
+  buf = malloc (BUFSIZ);
+  if (!buf)
+    return NULL;
+
+  ret = pmap (sctx, prop, buf, BUFSIZ - 1);
+
+  free (buf);
+
+  return ret;
+}
+
 int
 _gsasl_obsolete_callback (Gsasl * ctx, Gsasl_session * sctx,
 			  Gsasl_property prop)
 {
-  char buf[BUFSIZ];
-  size_t buflen = BUFSIZ - 1;
   int res;
 
   /* Call obsolete callbacks. */
@@ -2019,11 +2092,14 @@ _gsasl_obsolete_callback (Gsasl * ctx, Gsasl_session * sctx,
       {
 	Gsasl_server_callback_securid cb_securid
 	  = gsasl_server_callback_securid_get (sctx->ctx);
+#define MAX_SECURID 32 /* See RFC 2808. */
+	char buf[MAX_SECURID + 1];
+	size_t buflen = MAX_SECURID;
 	if (!cb_securid)
 	  break;
 	res = cb_securid (sctx, sctx->authid, sctx->authzid, sctx->passcode,
 			  sctx->pin, buf, &buflen);
-	if (buflen > 0 && buflen < BUFSIZ - 1)
+	if (buflen > 0 && buflen < MAX_SECURID)
 	  {
 	    buf[buflen] = '\0';
 	    gsasl_property_set (sctx, GSASL_SUGGESTED_PIN, buf);
@@ -2058,13 +2134,19 @@ _gsasl_obsolete_callback (Gsasl * ctx, Gsasl_session * sctx,
       {
 	Gsasl_server_callback_retrieve cb_retrieve
 	  = gsasl_server_callback_retrieve_get (sctx->ctx);
+	char *buf;
+	size_t buflen = BUFSIZ - 1;
 	if (!cb_retrieve)
 	  break;
+	buf = malloc (BUFSIZ);
+	if (!buf)
+	  return GSASL_MALLOC_ERROR;
 	res = cb_retrieve (sctx, sctx->authid, sctx->authzid,
 			   sctx->hostname, buf, &buflen);
 	if (res == GSASL_OK)
 	  gsasl_property_set_raw (sctx, GSASL_PASSWORD, buf, buflen);
 	/* FIXME else if (res == GSASL_TOO_SMALL_BUFFER)... */
+	free (buf);
 	return res;
 	break;
       }
