@@ -1,5 +1,5 @@
 /* version.c -- Version handling.
- * Copyright (C) 2002, 2004, 2005, 2006  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009  Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -22,27 +22,30 @@
 
 #include "internal.h"
 
-#include <strverscmp.h>
+#include <string.h>		/* for strverscmp */
 
 /**
  * gsasl_check_version:
- * @req_version: version string to compare with, or NULL.
+ * @req_version: version string to compare with, or %NULL.
  *
- * Check library version.
+ * Check GNU SASL Library version.
  *
  * See %GSASL_VERSION for a suitable @req_version string.
  *
- * Return value: Check that the the version of the library is at
+ * This function is one of few in the library that can be used without
+ * a successful call to gsasl_init().
+ *
+ * Return value: Check that the version of the library is at
  *   minimum the one given as a string in @req_version and return the
- *   actual version string of the library; return NULL if the
+ *   actual version string of the library; return %NULL if the
  *   condition is not met.  If %NULL is passed to this function no
  *   check is done and only the version string is returned.
  **/
 const char *
 gsasl_check_version (const char *req_version)
 {
-  if (!req_version || strverscmp (req_version, PACKAGE_VERSION) <= 0)
-    return PACKAGE_VERSION;
+  if (!req_version || strverscmp (req_version, GSASL_VERSION) <= 0)
+    return GSASL_VERSION;
 
   return NULL;
 }
