@@ -57,6 +57,9 @@ gsasl_finish (Gsasl_session * sctx)
   free (sctx->digest_md5_hashed_password);
   free (sctx->qops);
   free (sctx->qop);
+  free (sctx->scram_iter);
+  free (sctx->scram_salt);
+  free (sctx->scram_salted_password);
 
   free (sctx);
 }
