@@ -38,6 +38,8 @@
 #define HOSTNAME "hostname"
 #define REALM "realm"
 
+size_t i;
+
 static int
 callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
 {
@@ -93,6 +95,64 @@ callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
       rc = GSASL_NO_CALLBACK;
       break;
 
+    case GSASL_QOPS:
+      rc = GSASL_OK;
+      switch (i)
+	{
+	case 0:
+	  gsasl_property_set (sctx, prop, "qop-auth");
+	  break;
+
+	case 1:
+	  rc = GSASL_NO_CALLBACK;
+	  break;
+
+	case 2:
+	  gsasl_property_set (sctx, prop, "qop-int");
+	  break;
+
+	case 3:
+	  gsasl_property_set (sctx, prop, "qop-auth");
+	  break;
+
+	case 4:
+	  rc = GSASL_NO_CALLBACK;
+	  break;
+
+	default:
+	  break;
+	}
+      break;
+
+    case GSASL_QOP:
+      rc = GSASL_OK;
+      switch (i)
+	{
+	case 0:
+	  rc = GSASL_NO_CALLBACK;
+	  break;
+
+	case 1:
+	  gsasl_property_set (sctx, prop, "qop-auth");
+	  break;
+
+	case 2:
+	  gsasl_property_set (sctx, prop, "qop-int");
+	  break;
+
+	case 3:
+	  gsasl_property_set (sctx, prop, "qop-auth");
+	  break;
+
+	case 4:
+	  gsasl_property_set (sctx, prop, "qop-auth");
+	  break;
+
+	default:
+	  break;
+	}
+      break;
+
     default:
       fail ("Unknown callback property %d\n", prop);
       break;
@@ -108,7 +168,6 @@ doit (void)
   Gsasl_session *server = NULL, *client = NULL;
   char *s1, *s2;
   size_t s1len, s2len;
-  size_t i;
   int res;
 
   res = gsasl_init (&ctx);
@@ -118,6 +177,14 @@ doit (void)
       return;
     }
 
+  if (!gsasl_client_support_p (ctx, "DIGEST-MD5")
+      || !gsasl_server_support_p (ctx, "DIGEST-MD5"))
+    {
+      gsasl_done (ctx);
+      fail("No support for DIGEST-MD5.\n");
+      exit(77);
+    }
+
   gsasl_callback_set (ctx, callback);
 
   for (i = 0; i < 5; i++)
@@ -217,6 +284,102 @@ doit (void)
 
       gsasl_free (s1);
 
+      /* Encode data in client. */
+
+      res = gsasl_encode (client, "foo", 3, &s1, &s1len);
+      if (res != GSASL_OK)
+	{
+	  fail ("gsasl_encode failed (%d):\n%s\n", res,
+		gsasl_strerror (res));
+	  return;
+	}
+
+      if (debug)
+	{
+	  if (s1len == 3 && memcmp (s1, "foo", 3) == 0)
+	    printf ("C: %.*s\n", s1len, s1);
+	  else
+	    {
+	      char *out;
+	      size_t outlen;
+
+	      res = gsasl_base64_to (s1, s1len, &out, &outlen);
+	      if (res != GSASL_OK)
+		{
+		  fail ("gsasl_base64_to failed (%d):\n%s\n", res,
+			gsasl_strerror (res));
+		  return;
+		}
+
+	      printf ("C: %.*s\n", outlen, out);
+	      free (out);
+	    }
+	}
+
+      /* Decode data in server. */
+
+      res = gsasl_decode (server, s1, s1len, &s2, &s2len);
+      free (s1);
+      if (res != GSASL_OK)
+	{
+	  fail ("gsasl_decode failed (%d):\n%s\n", res,
+		gsasl_strerror (res));
+	  return;
+	}
+
+      if (debug)
+	printf ("S: %.*s\n", s2len, s2);
+
+      free (s2);
+
+      /* Encode data in server. */
+
+      res = gsasl_encode (server, "bar", 3, &s1, &s1len);
+      if (res != GSASL_OK)
+	{
+	  fail ("gsasl_encode(2) failed (%d):\n%s\n", res,
+		gsasl_strerror (res));
+	  return;
+	}
+
+      if (debug)
+	{
+	  if (s1len == 3 && memcmp (s1, "bar", 3) == 0)
+	    printf ("S: %.*s\n", s1len, s1);
+	  else
+	    {
+	      char *out;
+	      size_t outlen;
+
+	      res = gsasl_base64_to (s1, s1len, &out, &outlen);
+	      if (res != GSASL_OK)
+		{
+		  fail ("gsasl_base64_to(2) failed (%d):\n%s\n", res,
+			gsasl_strerror (res));
+		  return;
+		}
+
+	      printf ("S: %.*s\n", outlen, out);
+	      free (out);
+	    }
+	}
+
+      /* Decode data in client. */
+
+      res = gsasl_decode (client, s1, s1len, &s2, &s2len);
+      free (s1);
+      if (res != GSASL_OK)
+	{
+	  fail ("gsasl_decode failed (%d):\n%s\n", res,
+		gsasl_strerror (res));
+	  return;
+	}
+
+      if (debug)
+	printf ("C: %.*s\n", s2len, s2);
+
+      free (s2);
+
       if (debug)
 	printf ("\n");
 
